@@ -579,7 +579,8 @@ Create a Data::HexDump::Range object.
 		DISPLAY_RANGE_SIZE => 1,
 		DISPLAY_ASCII_DUMP => 1 ,
 		DISPLAY_HEX_DUMP => 1,
-		DISPLAY_DEC_DUMP => 1,
+		DISPLAY_HEXASCII_DUMP => 0,
+		DISPLAY_DEC_DUMP => 0,
 		COLOR_NAMES => {},
 		ORIENTATION => 'horizontal',
 		) ;
@@ -630,10 +631,19 @@ transformed  into the format that is internally used by the module. These are fo
 
 Default is B<ANSI> which allows for colors. Other formats are 'ASCII' and 'HTML'.
 
-=item * COLOR - String 'bw' or 'cycle'.
+=item * COLOR - String 'cycle', 'no_cycle', 'bw'
 
-Ranges for which no color has been defined, in 'ANSI' or 'HTML' format mode, will be rendered in
-black and white or with a color picked from a cyclic color list. Default is 'bw'.
+Controls the coloring policy of ranges
+
+=over 2
+
+=item * cycle: if the range definition contains a color does not define a color, a color is automatically assigned
+
+=item * no_cycle: only ranges with a color defined are colorized
+
+=item * bw: no color is used even if the range contains a color definition
+
+=back
 
 =item * OFFSET_FORMAT - String - 'hex' or 'dec'
 
@@ -668,6 +678,8 @@ in base 10. Default is 'hex'.
 
 =item * DISPLAY_HEX_DUMP - Boolean - If set, the hexadecimal dump column is displayed. Default is I<true>
 
+=item * DISPLAY_HEXASCII_DUMP - Boolean - If set, the comined hexadecimal and ASCII dump column is displayed. Default is I<false>
+
 =item * DISPLAY_DEC_DUMP - Boolean - If set, the decimall dump column is displayed. Default is I<false>
 
 =item * DISPLAY_BITFIELD_SOURCE - Boolean - if set an extra column indicating the source of bitfields is displayed
@@ -918,7 +930,7 @@ None so far.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright Nadim Khemir 2010.
+Copyright Nadim Khemir 2010-2012.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of either:
