@@ -28,6 +28,7 @@ use Readonly ;
 Readonly my $EMPTY_STRING => q{} ;
 
 use Carp qw(carp croak confess) ;
+use Text::Pluralize ;
 
 #-------------------------------------------------------------------------------
 
@@ -172,15 +173,16 @@ while (my $range = shift @collected_data_to_dump)
 		
 	$range->{SOURCE} = $range_source  if $range->{IS_BITFIELD} ;
 		
-	$range->{COLOR} = $self->get_default_color()  unless defined $range->{COLOR} ;
-	
+
 	if($range->{IS_BITFIELD}) 
 		{
+		$range->{COLOR} = $range_source->[1] ;
 		push @found_bitfields, $self->get_bitfield_lines($range) ;
-		
 		next ;
 		}
 	
+	$range->{COLOR} = $self->get_default_color($range->{COLOR}) ;
+	
 	if($room_left == $self->{DATA_WIDTH})
 		{
 		push @lines,  @found_bitfields ;
@@ -370,7 +372,7 @@ while (my $range = shift @collected_data_to_dump)
 		
 	# vertical mode
 		
-	$range->{COLOR} = $self->get_default_color()  unless defined $range->{COLOR} ;
+	$range->{COLOR} = $self->get_default_color($range->{COLOR}) ;
 	
 	$line = {} ;
 
@@ -428,7 +430,17 @@ while (my $range = shift @collected_data_to_dump)
 			sub 
 				{
 				my @bytes = unpack("(H2)*", pack("N", $data_length));
-				"@bytes bytes skipped" ;
+				pluralize("Skipped @bytes byte(s)", $data_length) ;
+				},
+			$range->{COLOR},
+			3 * $self->{DATA_WIDTH},
+			],
+			[
+			'HEXASCII_DUMP', 
+			sub 
+				{
+				my @bytes = unpack("(H2)*", pack("N", $data_length));
+				pluralize("Skipped @bytes byte(s)", $data_length) ;
 				},
 			$range->{COLOR},
 			3 * $self->{DATA_WIDTH},
@@ -437,14 +449,13 @@ while (my $range = shift @collected_data_to_dump)
 			'DEC_DUMP', 
 			sub 
 				{
-				my @values = map {sprintf '%03u', $_} unpack("(C3)*", pack("N", $data_length));
-				join(' ',  @values) . " skipped: $data_length bytes"  ;
+				pluralize("Skipped $data_length byte(s)", $data_length)  ;
 				},
 			$range->{COLOR},
 			4 * $self->{DATA_WIDTH}
 			],
 			['ASCII_DUMP', sub {$EMPTY_STRING}, $range->{COLOR}, $self->{DATA_WIDTH}],
-							['USER_INFORMATION', sub { sprintf '%-20.20s', $range->{USER_INFORMATION} || ''}, $range->{COLOR}, 20],
+			['USER_INFORMATION', sub { sprintf '%-20.20s', $range->{USER_INFORMATION} || ''}, $range->{COLOR}, 20],
 			)
 			{
 			my ($field_name, $field_data_formater, $color, $field_text_size) = @{$field_type} ;
@@ -573,11 +584,12 @@ my $max_range_name_size = $self->{MAXIMUM_RANGE_NAME_SIZE} ;
 		
 for my  $field_type 
 	(
-	['OFFSET', sub {exists $line->{OFFSET} ? '' : sprintf $self->{OFFSET_FORMAT}, $total_dumped_data}, undef, 0],
-	['BITFIELD_SOURCE', sub {exists $line->{BITFIELD_SOURCE} ? '' : ' ' x 8}, undef, 0],
+	['OFFSET', sub {exists $line->{OFFSET} ? '' : sprintf $self->{OFFSET_FORMAT}, $total_dumped_data}, $self->get_bg_color(), 0],
+	['BITFIELD_SOURCE', sub {exists $line->{BITFIELD_SOURCE} ? '' : ' ' x 8}, $self->get_bg_color(), 0],
 	['HEX_DUMP', sub {sprintf '%02x ' x $size_to_dump, @_}, $range->{COLOR}, 3],
 	['DEC_DUMP', sub {sprintf '%03u ' x $size_to_dump, @_}, $range->{COLOR}, 4],
 	['ASCII_DUMP', sub {sprintf '%c' x $size_to_dump, map{$_ < 30 ? ord('.') : $_ } @_}, $range->{COLOR}, 1],
+	['HEXASCII_DUMP', sub {sprintf q~%02x/%c ~ x $size_to_dump, map{$_ < 30 ? ($_, ord('.')) : ($_, $_) } @_}, $range->{COLOR}, 5],
 	['RANGE_NAME',sub {sprintf "%.${max_range_name_size}s", $range->{NAME}}, $range->{COLOR}, 0],
 	['RANGE_NAME', sub {', '}, undef, 0],
 	)
@@ -594,7 +606,7 @@ for my  $field_type
 
 		unless($visible)
 			{
-			if($field_name eq 'ASCII_DUMP' || $field_name eq 'HEX_DUMP'  || $field_name eq 'DEC_DUMP' )
+			if($field_name eq 'ASCII_DUMP' || $field_name eq 'HEX_DUMP'  || $field_name eq 'HEXASCII_DUMP'  || $field_name eq 'DEC_DUMP' )
 				{
 				$text = ' ' x length($text)
 				}
@@ -652,10 +664,11 @@ my $user_information_size = $self->{MAXIMUM_USER_INFORMATION_SIZE} ;
 for my  $field_type 
 	(
 	['RANGE_NAME',  sub {sprintf "%-${max_range_name_size}.${max_range_name_size}s", $range->{NAME} ; }, $range->{COLOR}, $max_range_name_size] ,
-	['OFFSET', sub {sprintf $self->{OFFSET_FORMAT}, $total_dumped_data}, undef, 8],
-	['CUMULATIVE_OFFSET', sub {sprintf $self->{OFFSET_FORMAT}, $dumped_data}, undef, 8],
+	['OFFSET', sub {sprintf $self->{OFFSET_FORMAT}, $total_dumped_data}, $self->get_bg_color(), 8],
+	['CUMULATIVE_OFFSET', sub {$dumped_data ? sprintf($self->{OFFSET_FORMAT}, $dumped_data) : ''}, $self->get_bg_color(), 8],
 	['BITFIELD_SOURCE', sub {'' x 8}, undef, 8],
 	['HEX_DUMP', sub {sprintf '%02x ' x $size_to_dump, @{$_[0]}}, $range->{COLOR}, 3 * $size_to_dump],
+        ['HEXASCII_DUMP', sub {sprintf q~%02x/%c ~ x $size_to_dump, map{$_ < 30 ? ($_, ord('.')) : ($_, $_) } @{ $_[0]}}, $range->{COLOR}, 5 * $size_to_dump],
 	['DEC_DUMP', sub {sprintf '%03u ' x $size_to_dump, @{ $_[0] }}, $range->{COLOR}, 4 * $size_to_dump],
 	['ASCII_DUMP', sub {sprintf '%c' x $size_to_dump, map{$_ < 30 ? ord('.') : $_ } @{$_[0]}}, $range->{COLOR}, $size_to_dump],
 	['USER_INFORMATION', sub { sprintf "%-${user_information_size}.${user_information_size}s", $range->{USER_INFORMATION} || ''}, $range->{COLOR}, $user_information_size],
@@ -672,7 +685,7 @@ for my  $field_type
 		
 		unless($visible)
 			{
-			if($field_name eq 'ASCII_DUMP' || $field_name eq 'HEX_DUMP'  || $field_name eq 'DEC_DUMP' )
+			if($field_name eq 'ASCII_DUMP' || $field_name eq 'HEX_DUMP'  || $field_name eq 'DEC_DUMP'  || $field_name eq 'HEXASCII_DUMP' )
 				{
 				$text = ' ' x length($text) ;
 				}
@@ -757,6 +770,32 @@ my %always_display_field = map {$_ => 1} qw(RANGE_NAME OFFSET CUMULATIVE_OFFSET
 my $bitfield_warning_displayed = 0 ;
 
 #~ print DumpTree {length => length($bitfield_description->{DATA}), offset => $offset, size => $size, BF => $bitfield_description} ;
+my $ascii_bitfield_dump_sub =
+	sub 
+	{
+	my ($binary, @binary , @chars) ;
+	
+	if($self->{BIT_ZERO_ON_LEFT})
+		{
+		@binary = split '', unpack("B*",  $_[0]->{DATA}) ;
+		splice(@binary, 0, $offset) ;
+		splice(@binary, $size) ;
+		}
+	else
+		{
+		@binary = split '', unpack("B*",  $_[0]->{DATA}) ;
+		splice(@binary, -$offset) unless $offset == 0 ;
+		@binary = splice(@binary, - $size) ;
+		}
+		
+	$binary = join('', @binary) ;
+	@chars = map{$_ < 30 ? '.' : chr($_) } unpack("C*", pack("B32", substr("0" x 32 . $binary, -32)));
+	
+	my $number_of_bytes = @binary > 24 ? 4 : @binary > 16 ? 3 : @binary > 8 ? 2 : 1 ;
+	splice @chars, 0 , (4 - $number_of_bytes), map {'-'} 1 .. (4 - $number_of_bytes) ;
+	
+	'.bitfield: '.  join('',  @chars) 
+	} ;
 
 for my  $field_type 
 	(
@@ -779,6 +818,7 @@ for my  $field_type
 			
 			$binary_dashed = '-' x $offset . $binary . '-' x (32 - ($size + $offset)) ;
 			$binary_dashed  = substr($binary_dashed , -32) ;
+			$binary_dashed = substr($binary_dashed, 0, 8) . ' ' . substr($binary_dashed, 8, 8) . ' ' .substr($binary_dashed, 16, 8) . ' ' .substr($binary_dashed, 24, 8) ;
 			}
 		else
 			{
@@ -790,6 +830,7 @@ for my  $field_type
 			
 			$binary_dashed = '-' x (32 - ($size + $offset)) . $binary . '-' x $offset  ;
 			$binary_dashed  = substr($binary_dashed , 0, 32) ;
+			$binary_dashed = substr($binary_dashed, 0, 8) . ' ' . substr($binary_dashed, 8, 8) . ' ' .substr($binary_dashed, 16, 8) . ' ' .substr($binary_dashed, 24, 8) ;
 			}
 		
 		my $bytes = $size > 24 ? 4 : $size > 16 ? 3 : $size > 8 ? 2 : 1 ;
@@ -799,10 +840,56 @@ for my  $field_type
 		my $number_of_bytes = @binary > 24 ? 4 : @binary > 16 ? 3 : @binary > 8 ? 2 : 1 ;
 		splice @bytes, 0 , (4 - $number_of_bytes), map {'--'} 1 .. (4 - $number_of_bytes) ;
 		
-		join(' ', @bytes) . '    ' . $binary_dashed;
+		join(' ', @bytes) . ' ' . $binary_dashed;
 		},
 		
 		undef, 3 * $self->{DATA_WIDTH}],
+	['HEXASCII_DUMP', 
+		sub 
+		{
+		my $ascii_bitfield_dump = $ascii_bitfield_dump_sub->(@_) ;
+	
+
+		my ($binary, @binary , $binary_dashed) ;
+		
+		if($self->{BIT_ZERO_ON_LEFT})
+			{
+			@binary = split '', unpack("B*",  $_[0]->{DATA}) ;
+			splice(@binary, 0, $offset) ;
+			splice(@binary, $size) ;
+			
+			$binary = join('', @binary) ;
+			
+			$binary_dashed = '-' x $offset . $binary . '-' x (32 - ($size + $offset)) ;
+			$binary_dashed  = substr($binary_dashed , -32) ;
+			$binary_dashed = substr($binary_dashed, 0, 8) . ' ' . substr($binary_dashed, 8, 8) . ' ' .substr($binary_dashed, 16, 8) . ' ' .substr($binary_dashed, 24, 8) ;
+			}
+		else
+			{
+			@binary = split '', unpack("B*",  $_[0]->{DATA}) ;
+			splice(@binary, -$offset) unless $offset == 0 ;
+			@binary = splice(@binary, - $size) ;
+			
+			$binary = join('',  @binary) ;
+			
+			$binary_dashed = '-' x (32 - ($size + $offset)) . $binary . '-' x $offset  ;
+			$binary_dashed  = substr($binary_dashed , 0, 32) ;
+			$binary_dashed = substr($binary_dashed, 0, 8) . ' ' . substr($binary_dashed, 8, 8) . ' ' .substr($binary_dashed, 16, 8) . ' ' .substr($binary_dashed, 24, 8) ;
+			}
+		
+		my $bytes = $size > 24 ? 4 : $size > 16 ? 3 : $size > 8 ? 2 : 1 ;
+		
+		my @bytes = unpack("(H2)*", pack("B32", substr("0" x 32 . $binary, -32)));
+		
+		my $number_of_bytes = @binary > 24 ? 4 : @binary > 16 ? 3 : @binary > 8 ? 2 : 1 ;
+		splice @bytes, 0 , (4 - $number_of_bytes), map {'--'} 1 .. (4 - $number_of_bytes) ;
+		
+		join(' ', @bytes) . '    ' . $binary_dashed . '     ' . $ascii_bitfield_dump ;
+
+	
+		},
+		
+		undef, 5 * $self->{DATA_WIDTH}],
 	['DEC_DUMP', 
 		sub 
 		{
@@ -836,35 +923,12 @@ for my  $field_type
 		$bitfield_description->{COLOR}, 4 * $self->{DATA_WIDTH}],
 		
 	['ASCII_DUMP',
-		sub 
-		{
-		my ($binary, @binary , @chars) ;
-		
-		if($self->{BIT_ZERO_ON_LEFT})
-			{
-			@binary = split '', unpack("B*",  $_[0]->{DATA}) ;
-			splice(@binary, 0, $offset) ;
-			splice(@binary, $size) ;
-			}
-		else
-			{
-			@binary = split '', unpack("B*",  $_[0]->{DATA}) ;
-			splice(@binary, -$offset) unless $offset == 0 ;
-			@binary = splice(@binary, - $size) ;
-			}
-			
-		$binary = join('', @binary) ;
-		@chars = map{$_ < 30 ? '.' : chr($_) } unpack("C*", pack("B32", substr("0" x 32 . $binary, -32)));
-		
-		my $number_of_bytes = @binary > 24 ? 4 : @binary > 16 ? 3 : @binary > 8 ? 2 : 1 ;
-		splice @chars, 0 , (4 - $number_of_bytes), map {'-'} 1 .. (4 - $number_of_bytes) ;
-		
-		'.bitfield: '.  join('',  @chars) 
-		},
-
+		$ascii_bitfield_dump_sub,
 		undef, $self->{DATA_WIDTH}],
 		
-		['USER_INFORMATION', sub { sprintf '%-20.20s', $_[0]->{USER_INFORMATION} || ''}, $bitfield_description->{COLOR}, 20],
+	['USER_INFORMATION', 
+		sub { sprintf '%-20.20s', $_[0]->{USER_INFORMATION} || ''}, 
+		$bitfield_description->{COLOR}, 20],
 		
 	)
 	{
@@ -958,7 +1022,6 @@ I<Exceptions> - None
 
 =cut
 
-
 my ($self, $split_data) = @_ ;
 
 unshift @{$split_data}, $self->get_information($split_data) ;
@@ -1044,6 +1107,15 @@ if($self->{DISPLAY_RULER})
 					last ;
 					} ;
 					
+				/HEXASCII_DUMP/ and do
+					{
+					$information .= $self->{OFFSET_FORMAT} =~ /x$/
+							? join '', map {sprintf '%x    ' , $ _ % 16} (0 .. $self->{DATA_WIDTH} - 1)
+							: join '', map {sprintf '%d    ' , $ _ % 10} (0 .. $self->{DATA_WIDTH} - 1) ;
+					$information .= ' ' ;
+					last ;
+					} ;
+					
 				/ASCII_DUMP/ and do
 					{
 					$information .= $self->{OFFSET_FORMAT} =~ /x$/
